@@ -30,11 +30,12 @@
 #include "ssl.h"
 
 enum connect_result
-tcp_connect(int fd, uint32_t addr_ip, uint16_t addr_port)
+tcp_connect(int fd, REQ * req_obj)
 {
 	struct linger li = { 0 };
 	int long_inet;
 	struct sockaddr_in adr_serv;
+	struct sockaddr_in6 adr_serv6;
 	int ret;
 	int val;
 
@@ -44,18 +45,40 @@ tcp_connect(int fd, uint32_t addr_ip, uint16_t addr_port)
 	setsockopt(fd, SOL_SOCKET, SO_LINGER, (char *) &li,
 		   sizeof (struct linger));
 
-	long_inet = sizeof (struct sockaddr_in);
-	memset(&adr_serv, 0, long_inet);
-	adr_serv.sin_family = AF_INET;
-	adr_serv.sin_port = addr_port;
-	adr_serv.sin_addr.s_addr = addr_ip;
-
 	/* Make socket non-block. */
 	val = fcntl(fd, F_GETFL, 0);
 	fcntl(fd, F_SETFL, val | O_NONBLOCK);
 
-	/* Call connect function. */
-	ret = connect(fd, (struct sockaddr *) &adr_serv, long_inet);
+	if(req_obj->dst){
+		if(req_obj->dst->ai_family == AF_INET6) {
+			long_inet = sizeof (struct sockaddr_in6);
+			memset(&adr_serv6, 0, long_inet);
+			adr_serv6.sin6_family = req_obj->dst->ai_family;
+			adr_serv6.sin6_port = req_obj->addr_port;
+			inet_pton(AF_INET6, req_obj->ipaddress, &adr_serv6.sin6_addr);
+
+			/* Call connect function. */
+			ret = connect(fd, (struct sockaddr *) &adr_serv6, long_inet);
+		} else {
+			long_inet = sizeof (struct sockaddr_in);
+			memset(&adr_serv, 0, long_inet);
+			adr_serv.sin_family = req_obj->dst->ai_family;
+			adr_serv.sin_port = req_obj->addr_port;
+			inet_pton(AF_INET, req_obj->ipaddress, &adr_serv.sin_addr);
+
+			/* Call connect function. */
+			ret = connect(fd, (struct sockaddr *) &adr_serv, long_inet);
+		}
+	} else {
+		long_inet = sizeof (struct sockaddr_in);
+		memset(&adr_serv, 0, long_inet);
+		adr_serv.sin_family = AF_INET;
+		adr_serv.sin_port = req_obj->addr_port;
+		inet_pton(AF_INET, req_obj->ipaddress, &adr_serv.sin_addr);
+
+		/* Call connect function. */
+		ret = connect(fd, (struct sockaddr *) &adr_serv, long_inet);
+	}
 
 	/* Immediate success */
 	if (ret == 0) {
@@ -75,7 +98,7 @@ tcp_connect(int fd, uint32_t addr_ip, uint16_t addr_port)
 }
 
 enum connect_result
-tcp_socket_state(int fd, thread_t * thread, uint32_t addr_ip, uint16_t addr_port,
+tcp_socket_state(int fd, thread_t * thread, char *ipaddress, uint16_t addr_port,
 		 int (*func) (thread_t *))
 {
 	int status;
@@ -86,7 +109,7 @@ tcp_socket_state(int fd, thread_t * thread, uint32_t addr_ip, uint16_t addr_port
 	/* Handle connection timeout */
 	if (thread->type == THREAD_WRITE_TIMEOUT) {
 		DBG("TCP connection timeout to [%s]:%d.\n",
-		    inet_ntop2(addr_ip), ntohs(addr_port));
+		    ipaddress, ntohs(addr_port));
 		close(thread->u.fd);
 		return connect_timeout;
 	}
@@ -100,7 +123,7 @@ tcp_socket_state(int fd, thread_t * thread, uint32_t addr_ip, uint16_t addr_port
 	/* Connection failed !!! */
 	if (ret) {
 		DBG("TCP connection failed to [%s]:%d.\n",
-		    inet_ntop2(addr_ip), ntohs(addr_port));
+		    ipaddress, ntohs(addr_port));
 		close(thread->u.fd);
 		return connect_error;
 	}
@@ -112,7 +135,7 @@ tcp_socket_state(int fd, thread_t * thread, uint32_t addr_ip, uint16_t addr_port
 	 */
 	if (status != 0) {
 		DBG("TCP connection to [%s]:%d still IN_PROGRESS.\n",
-		    inet_ntop2(addr_ip), ntohs(addr_port));
+		    ipaddress, ntohs(addr_port));
 
 		timer_min = timer_sub_now(thread->sands);
 		thread_add_write(thread->master, func, THREAD_ARG(thread)
@@ -156,19 +179,19 @@ tcp_check_thread(thread_t * thread)
 	int ret = 1;
 
 	sock_obj->status =
-	    tcp_socket_state(thread->u.fd, thread, req->addr_ip, req->addr_port,
+	    tcp_socket_state(thread->u.fd, thread, req->ipaddress, req->addr_port,
 			     tcp_check_thread);
 	switch (sock_obj->status) {
 	case connect_error:
 		DBG("Error connecting server [%s]:%d.\n",
-		    inet_ntop2(req->addr_ip), ntohs(req->addr_port));
+		    req->ipaddress, ntohs(req->addr_port));
 		thread_add_terminate_event(thread->master);
 		return -1;
 		break;
 
 	case connect_timeout:
 		DBG("Timeout connecting server [%s]:%d.\n",
-		    inet_ntop2(req->addr_ip), ntohs(req->addr_port));
+		    req->ipaddress, ntohs(req->addr_port));
 		thread_add_terminate_event(thread->master);
 		return -1;
 		break;
@@ -186,7 +209,7 @@ tcp_check_thread(thread_t * thread)
 						 http_request_thread, sock_obj, 0);
 			} else {
 				DBG("Connection trouble to: [%s]:%d.\n",
-				    inet_ntop2(req->addr_ip),
+				    req->ipaddress,
 				    ntohs(req->addr_port));
 				if (req->ssl)
 					ssl_printerr(SSL_get_error
@@ -206,12 +229,26 @@ tcp_connect_thread(thread_t * thread)
 {
 	SOCK *sock_obj = THREAD_ARG(thread);
 
-	if ((sock_obj->fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1) {
-		DBG("WEB connection fail to create socket.\n");
-		return 0;
+	if(req->dst){
+		if(req->dst->ai_family == AF_INET6) {
+			if ((sock_obj->fd = socket(AF_INET6, SOCK_STREAM, IPPROTO_TCP)) == -1) {
+				DBG("WEB connection fail to create socket.\n");
+				return 0;
+			}
+		} else {
+			if ((sock_obj->fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1) {
+				DBG("WEB connection fail to create socket.\n");
+				return 0;
+			}
+		}
+	} else {
+		if ((sock_obj->fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1) {
+			DBG("WEB connection fail to create socket.\n");
+			return 0;
+		}
 	}
 
-	sock->status = tcp_connect(sock_obj->fd, req->addr_ip, req->addr_port);
+	sock->status = tcp_connect(sock_obj->fd, req);
 
 	/* handle tcp connection status & register check worker thread */
 	tcp_connection_state(sock_obj->fd, sock_obj->status, thread, tcp_check_thread,
