@@ -28,6 +28,11 @@
 #include "main.h"
 #include "utils.h"
 #include "signals.h"
+#include <sys/types.h>
+#include <stdio.h>
+#include <string.h>
+#include <sys/socket.h>
+#include <netdb.h>
 
 /* global var */
 REQ *req = NULL;
@@ -55,6 +60,8 @@ signal_init(void)
 static void
 usage(const char *prog)
 {
+	enum feat_hashes i;
+
 	fprintf(stderr, VERSION_STRING);
 	fprintf(stderr,
 		"Usage:\n"
@@ -69,71 +76,54 @@ usage(const char *prog)
 		"  %s --port            -p       Use the specified remote server port.\n"
 		"  %s --url             -u       Use the specified remote server url.\n"
 		"  %s --use-virtualhost -V       Use the specified virtualhost in GET query.\n"
+		"  %s --hash            -H       Use the specified hash algorithm.\n"
 		"  %s --verbose         -v       Use verbose mode output.\n"
 		"  %s --help            -h       Display this short inlined help screen.\n"
 		"  %s --release         -r       Display the release number\n",
-		prog, prog, prog, prog, prog, prog, prog, prog);
+		prog, prog, prog, prog, prog, prog, prog, prog, prog);
+	fprintf(stderr, "\nSupported hash algorithms:\n");
+	for (i = hash_first; i < hash_guard; i++)
+		fprintf(stderr, "  %s%s\n",
+			hashes[i].id, i == hash_default ? " (default)": "");
 }
 
 /* Command line parser */
 static int
 parse_cmdline(int argc, char **argv, REQ * req_obj)
 {
-	poptContext context;
-	char *optarg = NULL;
 	int c;
+	enum feat_hashes i;
+	struct addrinfo hint, *res = NULL;
+	int ret;
+	void *ptr;
 
-	struct poptOption options_table[] = {
-		{"release", 'r', POPT_ARG_NONE, NULL, 'r'},
-		{"help", 'h', POPT_ARG_NONE, NULL, 'h'},
-		{"verbose", 'v', POPT_ARG_NONE, NULL, 'v'},
-		{"use-ssl", 'S', POPT_ARG_NONE, NULL, 'S'},
-		{"server", 's', POPT_ARG_STRING, &optarg, 's'},
-		{"port", 'p', POPT_ARG_STRING, &optarg, 'p'},
-		{"url", 'u', POPT_ARG_STRING, &optarg, 'u'},
-		{"use-virtualhost", 'V', POPT_ARG_STRING, &optarg, 'V'},
-		{NULL, 0, 0, NULL, 0}
+	memset(&hint, '\0', sizeof hint);
+
+	hint.ai_family = PF_UNSPEC;
+	hint.ai_flags = AI_NUMERICHOST;
+
+	struct option long_options[] = {
+		{"release",         no_argument,       0, 'r'},
+		{"help",            no_argument,       0, 'h'},
+		{"verbose",         no_argument,       0, 'v'},
+		{"use-ssl",         no_argument,       0, 'S'},
+		{"server",          required_argument, 0, 's'},
+		{"hash",            required_argument, 0, 'H'},
+		{"use-virtualhost", required_argument, 0, 'V'},
+		{"port",            required_argument, 0, 'p'},
+		{"url",             required_argument, 0, 'u'},
+		{0, 0, 0, 0}
 	};
 
 	/* Parse the command line arguments */
-	context =
-	    poptGetContext(PROG, argc, (const char **) argv, options_table, 0);
-	if ((c = poptGetNextOpt(context)) < 0) {
-		usage(argv[0]);
-		return CMD_LINE_ERROR;
-	}
-
-	/* The first option car */
-	switch (c) {
-	case 'r':
-		fprintf(stderr, VERSION_STRING);
-		break;
-	case 'h':
-		usage(argv[0]);
-		break;
-	case 'v':
-		req_obj->verbose = 1;
-		break;
-	case 'S':
-		req_obj->ssl = 1;
-		break;
-	case 's':
-		if (!inet_ston(optarg, &req_obj->addr_ip)) {
-			fprintf(stderr, "server should be an IP, not %s\n", optarg);
-			return CMD_LINE_ERROR;
-		}
-		break;
-	case 'V':
-		req_obj->vhost = optarg;
-		break;
-	default:
-		usage(argv[0]);
-		return CMD_LINE_ERROR;
-	}
-
-	/* the others */
-	while ((c = poptGetNextOpt(context)) >= 0) {
+	while ((c = getopt_long (argc, argv, "rhvSs:H:V:p:u:", long_options, NULL)) != EOF) {
 		switch (c) {
+		case 'r':
+			fprintf(stderr, VERSION_STRING);
+			break;
+		case 'h':
+			usage(argv[0]);
+			break;
 		case 'v':
 			req_obj->verbose = 1;
 			break;
@@ -141,9 +131,33 @@ parse_cmdline(int argc, char **argv, REQ * req_obj)
 			req_obj->ssl = 1;
 			break;
 		case 's':
-			if (!inet_ston(optarg, &req_obj->addr_ip)) {
+			if ((ret = getaddrinfo(optarg, NULL, &hint, &res)) != 0){
 				fprintf(stderr, "server should be an IP, not %s\n", optarg);
 				return CMD_LINE_ERROR;
+			} else {
+				if(res->ai_family == AF_INET) {
+					req_obj->dst = res;
+					ptr = &((struct sockaddr_in *) res->ai_addr)->sin_addr;
+					inet_ntop (res->ai_family, ptr, req_obj->ipaddress, INET6_ADDRSTRLEN);
+				} else if (res->ai_family == AF_INET6) {
+					req_obj->dst = res;
+					ptr = &((struct sockaddr_in6 *) res->ai_addr)->sin6_addr;
+					inet_ntop (res->ai_family, ptr, req_obj->ipaddress, INET6_ADDRSTRLEN);
+				} else {
+					fprintf(stderr, "server should be an IP, not %s\n", optarg);
+					return CMD_LINE_ERROR;
+				}
+			}
+			break;
+		case 'H':
+			for (i = hash_first; i < hash_guard; i++)
+				if (!strcasecmp(optarg, hashes[i].id)) {
+					req_obj->hash = i;
+					break;
+				}
+			if (i == hash_guard) {
+				fprintf(stderr, "unknown hash algoritm: %s\n", optarg);
+				return CMD_LINE_ERROR;
 			}
 			break;
 		case 'V':
@@ -162,14 +176,14 @@ parse_cmdline(int argc, char **argv, REQ * req_obj)
 	}
 
 	/* check unexpected arguments */
-	if ((optarg = (char *) poptGetArg(context))) {
-		fprintf(stderr, "unexpected argument %s\n", optarg);
+	if (optind < argc) {
+		fprintf(stderr, "Unexpected argument(s): ");
+		while (optind < argc)
+			printf("%s ", argv[optind++]);
+		printf("\n");
 		return CMD_LINE_ERROR;
 	}
 
-	/* free the allocated context */
-	poptFreeContext(context);
-
 	return CMD_LINE_SUCCESS;
 }
 
@@ -177,22 +191,35 @@ int
 main(int argc, char **argv)
 {
 	thread_t thread;
+	char *url_default = malloc(2);
+	url_default[0] = '/';
+	url_default[1] = '\0';
 
 	/* Allocate the room */
 	req = (REQ *) MALLOC(sizeof (REQ));
 
+	/* Preset (potentially) non-zero defaults */
+	req->hash = hash_default;
+
 	/* Command line parser */
 	if (!parse_cmdline(argc, argv, req)) {
+		FREE(url_default);
 		FREE(req);
 		exit(0);
 	}
 
 	/* Check minimum configuration need */
-	if (!req->addr_ip && !req->addr_port && !req->url) {
+	if (!req->dst && !req->addr_port && !req->url) {
+		FREE(url_default);
+		freeaddrinfo(req->dst);
 		FREE(req);
 		exit(0);
 	}
 
+	if(!req->url){
+		req->url = url_default;
+	}
+
 	/* Init the reference timer */
 	req->ref_time = timer_tol(timer_now());
 	DBG("Reference timer = %lu\n", req->ref_time);
@@ -223,11 +250,13 @@ main(int argc, char **argv)
 	/* Finalize output informations */
 	if (req->verbose)
 		printf("Global response time for [%s] =%lu\n",
-		       req->url, req->response_time - req->ref_time);
+			    req->url, req->response_time - req->ref_time);
 
 	/* exit cleanly */
+	FREE(url_default);
 	SSL_CTX_free(req->ctx);
 	free_sock(sock);
+	freeaddrinfo(req->dst);
 	FREE(req);
 	exit(0);
 }
