@@ -90,7 +90,10 @@ static void
 start_check(void)
 {
 	/* Initialize sub-system */
-	ipvs_start();
+	if (ipvs_start() != IPVS_SUCCESS) {
+		stop_check();
+		return;
+	}
 	init_checkers_queue();
 #ifdef _WITH_VRRP_
 	init_interface_queue();
@@ -120,8 +123,10 @@ start_check(void)
 	}
 
 	/* Processing differential configuration parsing */
-	if (reload)
+	if (reload) {
 		clear_diff_services();
+		copy_srv_states();
+	}
 
 	/* Initialize IPVS topology */
 	if (!init_services()) {
@@ -178,6 +183,8 @@ reload_check_thread(thread_t * thread)
 	/* set the reloading flag */
 	SET_RELOAD;
 
+	log_message(LOG_INFO, "Got SIGHUP, reloading checker configuration");
+
 	/* Signals handling */
 	signal_reset();
 	signal_handler_destroy();
@@ -227,8 +234,13 @@ check_respawn_thread(thread_t * thread)
 	}
 
 	/* We catch a SIGCHLD, handle it */
-	log_message(LOG_ALERT, "Healthcheck child process(%d) died: Respawning", pid);
-	start_check_child();
+	if (!(debug & 64)) {
+		log_message(LOG_ALERT, "Healthcheck child process(%d) died: Respawning", pid);
+		start_check_child();
+	} else {
+		log_message(LOG_ALERT, "Healthcheck child process(%d) died: Exiting", pid);
+		raise(SIGTERM);
+	}
 	return 0;
 }
 
