@@ -86,10 +86,10 @@ build_ssl_ctx(void)
 	}
 
 	/* Load our keys and certificates */
-	if (check_data->ssl->keyfile)
+	if (check_data->ssl->certfile)
 		if (!
 		    (SSL_CTX_use_certificate_chain_file
-		     (ssl->ctx, check_data->ssl->keyfile))) {
+		     (ssl->ctx, check_data->ssl->certfile))) {
 			log_message(LOG_INFO,
 			       "SSL error : Cant load certificate file...");
 			return 0;
@@ -192,8 +192,8 @@ ssl_connect(thread_t * thread, int new_req)
 {
 	checker_t *checker = THREAD_ARG(thread);
 	http_checker_t *http_get_check = CHECKER_ARG(checker);
-	http_arg_t *http_arg = HTTP_ARG(http_get_check);
-	request_t *req = HTTP_REQ(http_arg);
+	http_t *http = HTTP_ARG(http_get_check);
+	request_t *req = HTTP_REQ(http);
 	int ret = 0;
 	int val = 0;
 
@@ -242,8 +242,9 @@ ssl_read_thread(thread_t * thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
 	http_checker_t *http_get_check = CHECKER_ARG(checker);
-	http_arg_t *http_arg = HTTP_ARG(http_get_check);
-	request_t *req = HTTP_REQ(http_arg);
+	http_t *http = HTTP_ARG(http_get_check);
+	request_t *req = HTTP_REQ(http);
+	unsigned timeout = checker->co->connection_to;
 	unsigned char digest[16];
 	int r = 0;
 	int val;
@@ -269,7 +270,7 @@ ssl_read_thread(thread_t * thread)
 	if (req->error == SSL_ERROR_WANT_READ) {
 		 /* async read unfinished */ 
 		thread_add_read(thread->master, ssl_read_thread, checker,
-				thread->u.fd, http_get_check->connection_to);
+				thread->u.fd, timeout);
 	} else if (r > 0 && req->error == 0) {
 		/* Handle response stream */
 		http_process_response(req, r);
@@ -279,7 +280,7 @@ ssl_read_thread(thread_t * thread)
 		 * Register itself to not perturbe global I/O multiplexer.
 		 */
 		thread_add_read(thread->master, ssl_read_thread, checker,
-				thread->u.fd, http_get_check->connection_to);
+				thread->u.fd, timeout);
 	} else if (req->error) {
 
 		/* All the SSL streal has been parsed */
