@@ -17,7 +17,7 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2011 Alexandre Cassen, <acassen@linux-vs.org>
+ * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #include "check_tcp.h"
@@ -36,54 +36,22 @@ int tcp_connect_thread(thread_t *);
 void
 free_tcp_check(void *data)
 {
-	tcp_checker_t *tcp_chk = CHECKER_DATA(data);
-
-	FREE(tcp_chk);
+	FREE(CHECKER_CO(data));
 	FREE(data);
 }
 
 void
 dump_tcp_check(void *data)
 {
-	tcp_checker_t *tcp_chk = CHECKER_DATA(data);
-
 	log_message(LOG_INFO, "   Keepalive method = TCP_CHECK");
-	log_message(LOG_INFO, "   Connection port = %d", ntohs(inet_sockaddrport(&tcp_chk->dst)));
-	if (tcp_chk->bindto.ss_family)
-		log_message(LOG_INFO, "   Bind to = %s", inet_sockaddrtos(&tcp_chk->bindto));
-	log_message(LOG_INFO, "   Connection timeout = %d", tcp_chk->connection_to/TIMER_HZ);
+	dump_conn_opts (CHECKER_GET_CO());
 }
 
 void
-tcp_check_handler(vector strvec)
+tcp_check_handler(vector_t *strvec)
 {
-	tcp_checker_t *tcp_chk = (tcp_checker_t *) MALLOC(sizeof (tcp_checker_t));
-
 	/* queue new checker */
-	checker_set_dst(&tcp_chk->dst);
-	queue_checker(free_tcp_check, dump_tcp_check, tcp_connect_thread, tcp_chk);
-}
-
-void
-connect_port_handler(vector strvec)
-{
-	tcp_checker_t *tcp_chk = CHECKER_GET();
-
-	checker_set_dst_port(&tcp_chk->dst, htons(CHECKER_VALUE_INT(strvec)));
-}
-
-void
-bind_handler(vector strvec)
-{
-	tcp_checker_t *tcp_chk = CHECKER_GET();
-	inet_stosockaddr(VECTOR_SLOT(strvec, 1), 0, &tcp_chk->bindto);
-}
-
-void
-connect_timeout_handler(vector strvec)
-{
-	tcp_checker_t *tcp_chk = CHECKER_GET();
-	tcp_chk->connection_to = CHECKER_VALUE_INT(strvec) * TIMER_HZ;
+	queue_checker(free_tcp_check, dump_tcp_check, tcp_connect_thread, NULL, CHECKER_NEW_CO());
 }
 
 void
@@ -91,9 +59,8 @@ install_tcp_check_keyword(void)
 {
 	install_keyword("TCP_CHECK", &tcp_check_handler);
 	install_sublevel();
-	install_keyword("connect_port", &connect_port_handler);
-	install_keyword("bindto", &bind_handler);
-	install_keyword("connect_timeout", &connect_timeout_handler);
+	install_connect_keywords();
+	install_keyword("warmup", &warmup_handler);
 	install_sublevel_end();
 }
 
@@ -101,11 +68,9 @@ int
 tcp_check_thread(thread_t * thread)
 {
 	checker_t *checker;
-	tcp_checker_t *tcp_check;
 	int status;
 
 	checker = THREAD_ARG(thread);
-	tcp_check = CHECKER_ARG(checker);
 
 	status = tcp_socket_state(thread->u.fd, thread, tcp_check_thread);
 
@@ -116,9 +81,8 @@ tcp_check_thread(thread_t * thread)
 		close(thread->u.fd);
 
 		if (!svr_checker_up(checker->id, checker->rs)) {
-			log_message(LOG_INFO, "TCP connection to [%s]:%d success."
-					    , inet_sockaddrtos(&tcp_check->dst)
-					    , ntohs(inet_sockaddrport(&tcp_check->dst)));
+			log_message(LOG_INFO, "TCP connection to %s success."
+					, FMT_TCP_RS(checker));
 			smtp_alert(checker->rs, NULL, NULL,
 				   "UP",
 				   "=> TCP CHECK succeed on service <=");
@@ -130,9 +94,8 @@ tcp_check_thread(thread_t * thread)
 	} else {
 
 		if (svr_checker_up(checker->id, checker->rs)) {
-			log_message(LOG_INFO, "TCP connection to [%s]:%d failed !!!"
-					    , inet_sockaddrtos(&tcp_check->dst)
-					    , ntohs(inet_sockaddrport(&tcp_check->dst)));
+			log_message(LOG_INFO, "TCP connection to %s failed !!!"
+					, FMT_TCP_RS(checker));
 			smtp_alert(checker->rs, NULL, NULL,
 				   "DOWN",
 				   "=> TCP CHECK failed on service <=");
@@ -154,7 +117,7 @@ int
 tcp_connect_thread(thread_t * thread)
 {
 	checker_t *checker = THREAD_ARG(thread);
-	tcp_checker_t *tcp_check = CHECKER_ARG(checker);
+	conn_opts_t *co = checker->co;
 	int fd;
 	int status;
 
@@ -168,20 +131,24 @@ tcp_connect_thread(thread_t * thread)
 		return 0;
 	}
 
-	if ((fd = socket(tcp_check->dst.ss_family, SOCK_STREAM, IPPROTO_TCP)) == -1) {
-		DBG("TCP connect fail to create socket.");
+	if ((fd = socket(co->dst.ss_family, SOCK_STREAM, IPPROTO_TCP)) == -1) {
+		log_message(LOG_INFO, "TCP connect fail to create socket. Rescheduling.");
+		thread_add_timer(thread->master, tcp_connect_thread, checker,
+				checker->vs->delay_loop);
+
 		return 0;
 	}
 
-	status = tcp_bind_connect(fd, &tcp_check->dst, &tcp_check->bindto);
-	if (status == connect_error) {
-		thread_add_timer(thread->master, tcp_connect_thread, checker,
-				 checker->vs->delay_loop);
-		return 0;
-	}
+	status = tcp_bind_connect(fd, co);
 
 	/* handle tcp connection status & register check worker thread */
-	tcp_connection_state(fd, status, thread, tcp_check_thread,
-			     tcp_check->connection_to);
+	if(tcp_connection_state(fd, status, thread, tcp_check_thread,
+			co->connection_to)) {
+		close(fd);
+		log_message(LOG_INFO, "TCP socket bind failed. Rescheduling.");
+		thread_add_timer(thread->master, tcp_connect_thread, checker,
+				checker->vs->delay_loop);
+	}
+
 	return 0;
 }
