@@ -34,11 +34,11 @@ static int string_to_number(const char *, int, int);
 static int modprobe_ipvs(void);
 
 /* fetch virtual server group from group name */
-virtual_server_group *
+virtual_server_group_t *
 ipvs_get_group_by_name(char *gname, list l)
 {
 	element e;
-	virtual_server_group *vsg;
+	virtual_server_group_t *vsg;
 
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
 		vsg = ELEMENT_DATA(e);
@@ -119,7 +119,7 @@ ipvs_syncd_cmd(int cmd, char *ifname, int state, int syncid)
 
 /* IPVS group range rule */
 static int
-ipvs_group_range_cmd(int cmd, virtual_server_group_entry *vsg_entry)
+ipvs_group_range_cmd(int cmd, virtual_server_group_entry_t *vsg_entry)
 {
 	uint32_t addr_ip;
 	int err = 0;
@@ -140,10 +140,10 @@ ipvs_group_range_cmd(int cmd, virtual_server_group_entry *vsg_entry)
 
 /* set IPVS group rules */
 static int
-ipvs_group_cmd(int cmd, list vs_group, real_server * rs, char * vsgname)
+ipvs_group_cmd(int cmd, list vs_group, real_server_t * rs, char * vsgname)
 {
-	virtual_server_group *vsg = ipvs_get_group_by_name(vsgname, vs_group);
-	virtual_server_group_entry *vsg_entry;
+	virtual_server_group_t *vsg = ipvs_get_group_by_name(vsgname, vs_group);
+	virtual_server_group_entry_t *vsg_entry;
 	list l;
 	element e;
 	int err = 1;
@@ -198,7 +198,7 @@ ipvs_group_cmd(int cmd, list vs_group, real_server * rs, char * vsgname)
 
 /* Fill IPVS rule with root vs infos */
 void
-ipvs_set_rule(int cmd, virtual_server * vs, real_server * rs)
+ipvs_set_rule(int cmd, virtual_server_t * vs, real_server_t * rs)
 {
 	/* Clean up target rule */
 	memset(urule, 0, sizeof (struct ip_vs_rule_user));
@@ -210,9 +210,8 @@ ipvs_set_rule(int cmd, virtual_server * vs, real_server * rs)
 	urule->protocol = vs->service_type;
 
 	if (!parse_timeout(vs->timeout_persistence, &urule->timeout))
-		log_message(LOG_INFO, "IPVS : Virtual service [%s]:%d illegal timeout."
-				    , inet_ntop2(inet_sockaddrip4(&vs->addr))
-				    , ntohs(inet_sockaddrport(&vs->addr)));
+		log_message(LOG_INFO, "IPVS : Virtual service %s illegal timeout."
+					, FMT_VS(vs));
 
 	if (urule->timeout != 0 || vs->granularity_persistence)
 		urule->vs_flags = IP_VS_SVC_F_PERSISTENT;
@@ -235,7 +234,7 @@ ipvs_set_rule(int cmd, virtual_server * vs, real_server * rs)
 
 /* Set/Remove a RS from a VS */
 int
-ipvs_cmd(int cmd, list vs_group, virtual_server * vs, real_server * rs)
+ipvs_cmd(int cmd, list vs_group, virtual_server_t * vs, real_server_t * rs)
 {
 	int err = 0;
 
@@ -276,9 +275,9 @@ ipvs_cmd(int cmd, list vs_group, virtual_server * vs, real_server * rs)
 
 /* Remove a specific vs group entry */
 int
-ipvs_group_remove_entry(virtual_server *vs, virtual_server_group_entry *vsge)
+ipvs_group_remove_entry(virtual_server_t *vs, virtual_server_group_entry_t *vsge)
 {
-	real_server *rs;
+	real_server_t *rs;
 	int err = 0;
 	element e;
 	list l = vs->rs;
@@ -421,7 +420,7 @@ ipvs_syncd_cmd(int cmd, char *ifname, int state, int syncid)
 
 /* IPVS group range rule */
 static void
-ipvs_group_range_cmd(int cmd, virtual_server_group_entry *vsg_entry)
+ipvs_group_range_cmd(int cmd, virtual_server_group_entry_t *vsg_entry)
 {
 	uint32_t addr_ip, ip;
 
@@ -455,10 +454,10 @@ ipvs_group_range_cmd(int cmd, virtual_server_group_entry *vsg_entry)
 
 /* set IPVS group rules */
 static void
-ipvs_group_cmd(int cmd, list vs_group, real_server * rs, virtual_server * vs)
+ipvs_group_cmd(int cmd, list vs_group, real_server_t * rs, virtual_server_t * vs)
 {
-	virtual_server_group *vsg = ipvs_get_group_by_name(vs->vsgname, vs_group);
-	virtual_server_group_entry *vsg_entry;
+	virtual_server_group_t *vsg = ipvs_get_group_by_name(vs->vsgname, vs_group);
+	virtual_server_group_entry_t *vsg_entry;
 	list l;
 	element e;
 
@@ -495,7 +494,7 @@ ipvs_group_cmd(int cmd, list vs_group, real_server * rs, virtual_server * vs)
 		srule->af = AF_INET;
 		/* Need to get address family from first real server */
 		if (vs->rs && !LIST_ISEMPTY(vs->rs) &&
-		    (((real_server *)ELEMENT_DATA(LIST_HEAD(vs->rs)))->addr.ss_family == AF_INET6)) {
+		    (((real_server_t *)ELEMENT_DATA(LIST_HEAD(vs->rs)))->addr.ss_family == AF_INET6)) {
 			srule->af = AF_INET6;
 			srule->netmask = 128;
 		}
@@ -524,7 +523,7 @@ ipvs_group_cmd(int cmd, list vs_group, real_server * rs, virtual_server * vs)
 
 /* Fill IPVS rule with root vs infos */
 void
-ipvs_set_rule(int cmd, virtual_server * vs, real_server * rs)
+ipvs_set_rule(int cmd, virtual_server_t * vs, real_server_t * rs)
 {
 	/* Clean target rule */
 	memset(drule, 0, sizeof(ipvs_dest_t));
@@ -538,12 +537,15 @@ ipvs_set_rule(int cmd, virtual_server * vs, real_server * rs)
 	srule->protocol = vs->service_type;
 
 	if (!parse_timeout(vs->timeout_persistence, &srule->timeout))
-		log_message(LOG_INFO, "IPVS : Virtual service [%s]:%d illegal timeout."
-				    , inet_ntop2(inet_sockaddrip4(&vs->addr))
-				    , ntohs(inet_sockaddrport(&vs->addr)));
+		log_message(LOG_INFO, "IPVS : Virtual service %s illegal timeout."
+				    , FMT_VS(vs));
 
 	if (srule->timeout != 0 || vs->granularity_persistence)
-		srule->flags = IP_VS_SVC_F_PERSISTENT;
+		srule->flags |= IP_VS_SVC_F_PERSISTENT;
+
+	/* Only for UDP services */
+	if (vs->ops == 1 && srule->protocol == IPPROTO_UDP)
+		srule->flags |= IP_VS_SVC_F_ONEPACKET;
 
 	if (cmd == IP_VS_SO_SET_ADD || cmd == IP_VS_SO_SET_DEL)
 		if (vs->granularity_persistence)
@@ -568,7 +570,7 @@ ipvs_set_rule(int cmd, virtual_server * vs, real_server * rs)
 
 /* Set/Remove a RS from a VS */
 int
-ipvs_cmd(int cmd, list vs_group, virtual_server * vs, real_server * rs)
+ipvs_cmd(int cmd, list vs_group, virtual_server_t * vs, real_server_t * rs)
 {
 	/* Allocate the room */
 	memset(srule, 0, sizeof(ipvs_service_t));
@@ -596,7 +598,7 @@ ipvs_cmd(int cmd, list vs_group, virtual_server * vs, real_server * rs)
 			srule->af = AF_INET;
 			/* Need to get address family from first real server */
 			if (vs->rs && !LIST_ISEMPTY(vs->rs) &&
-			    (((real_server *)ELEMENT_DATA(LIST_HEAD(vs->rs)))->addr.ss_family == AF_INET6)) {
+			    (((real_server_t *)ELEMENT_DATA(LIST_HEAD(vs->rs)))->addr.ss_family == AF_INET6)) {
 				srule->af = AF_INET6;
 				srule->netmask = 128;
 			}
@@ -619,9 +621,9 @@ ipvs_cmd(int cmd, list vs_group, virtual_server * vs, real_server * rs)
 
 /* Remove a specific vs group entry */
 int
-ipvs_group_remove_entry(virtual_server *vs, virtual_server_group_entry *vsge)
+ipvs_group_remove_entry(virtual_server_t *vs, virtual_server_group_entry_t *vsge)
 {
-	real_server *rs;
+	real_server_t *rs;
 	element e;
 	list l = vs->rs;
 
@@ -682,12 +684,12 @@ ipvs_group_remove_entry(virtual_server *vs, virtual_server_group_entry *vsge)
    statistics of real servers. The update is only done if we need
    refreshing. */
 void
-ipvs_update_stats(virtual_server *vs)
+ipvs_update_stats(virtual_server_t *vs)
 {
 	element e, ge = NULL;
-	real_server *rs;
-	virtual_server_group *vsg = NULL;
-	virtual_server_group_entry *vsg_entry = NULL;
+	real_server_t *rs;
+	virtual_server_group_t *vsg = NULL;
+	virtual_server_group_entry_t *vsg_entry = NULL;
 	uint32_t addr_ip = 0;
 	union nf_inet_addr nfaddr;
 	ipvs_service_entry_t * serv = NULL;
