@@ -31,12 +31,15 @@
   #include "check_snmp.h"
 #endif
 
+/* out-of-order functions declarations */
+static void update_quorum_state(virtual_server_t * vs);
+
 /* Returns the sum of all RS weight in a virtual server. */
 long unsigned
-weigh_live_realservers(virtual_server * vs)
+weigh_live_realservers(virtual_server_t * vs)
 {
 	element e;
-	real_server *svr;
+	real_server_t *svr;
 	long unsigned count = 0;
 
 	for (e = LIST_HEAD(vs->rs); e; ELEMENT_NEXT(e)) {
@@ -49,15 +52,18 @@ weigh_live_realservers(virtual_server * vs)
 
 /* Remove a realserver IPVS rule */
 static int
-clear_service_rs(list vs_group, virtual_server * vs, list l)
+clear_service_rs(list vs_group, virtual_server_t * vs, list l)
 {
 	element e;
-	real_server *rs;
-	char rsip[INET6_ADDRSTRLEN];
+	real_server_t *rs;
+	long unsigned weight_sum;
 
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
 		rs = ELEMENT_DATA(e);
 		if (ISALIVE(rs)) {
+			log_message(LOG_INFO, "Removing service %s from VS %s"
+						, FMT_RS(rs)
+						, FMT_VS(vs));
 			if (!ipvs_cmd(LVS_CMD_DEL_DEST, vs_group, vs, rs))
 				return 0;
 			UNSET_ALIVE(rs);
@@ -68,12 +74,10 @@ clear_service_rs(list vs_group, virtual_server * vs, list l)
 			 * all the way down the exit, as necessary.
 			 */
 			if (rs->notify_down) {
-				log_message(LOG_INFO, "Executing [%s] for service [%s]:%d in VS [%s]:%d"
+				log_message(LOG_INFO, "Executing [%s] for service %s in VS %s"
 						    , rs->notify_down
-						    , inet_sockaddrtos2(&rs->addr, rsip)
-						    , ntohs(inet_sockaddrport(&rs->addr))
-						    , (vs->vsgname) ? vs->vsgname : inet_sockaddrtos(&vs->addr)
-						    , ntohs(inet_sockaddrport(&vs->addr)));
+						    , FMT_RS(rs)
+						    , FMT_VS(vs));
 				notify_exec(rs->notify_down);
 			}
 #ifdef _WITH_SNMP_
@@ -84,14 +88,16 @@ clear_service_rs(list vs_group, virtual_server * vs, list l)
 			 * we don't push in a sorry server then, hence the regression
 			 * is intended.
 			 */
-			if (vs->quorum_state == UP &&
-			    weigh_live_realservers(vs) < vs->quorum - vs->hysteresis) {
+			weight_sum = weigh_live_realservers(vs);
+			if (vs->quorum_state == UP && (
+				!weight_sum ||
+				weight_sum < vs->quorum - vs->hysteresis)
+			) {
 				vs->quorum_state = DOWN;
 				if (vs->quorum_down) {
-					log_message(LOG_INFO, "Executing [%s] for VS [%s]:%d"
+					log_message(LOG_INFO, "Executing [%s] for VS %s"
 							    , vs->quorum_down
-							    , (vs->vsgname) ? vs->vsgname : inet_sockaddrtos(&vs->addr)
-							    , ntohs(inet_sockaddrport(&vs->addr)));
+							    , FMT_VS(vs));
 					notify_exec(vs->quorum_down);
 				}
 #ifdef _WITH_SNMP_
@@ -106,7 +112,7 @@ clear_service_rs(list vs_group, virtual_server * vs, list l)
 
 /* Remove a virtualserver IPVS rule */
 static int
-clear_service_vs(list vs_group, virtual_server * vs)
+clear_service_vs(list vs_group, virtual_server_t * vs)
 {
 	/* Processing real server queue */
 	if (!LIST_ISEMPTY(vs->rs)) {
@@ -132,7 +138,7 @@ clear_services(void)
 {
 	element e;
 	list l = check_data->vs;
-	virtual_server *vs;
+	virtual_server_t *vs;
 
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
 		vs = ELEMENT_DATA(e);
@@ -144,10 +150,10 @@ clear_services(void)
 
 /* Set a realserver IPVS rules */
 static int
-init_service_rs(virtual_server * vs)
+init_service_rs(virtual_server_t * vs)
 {
 	element e;
-	real_server *rs;
+	real_server_t *rs;
 
 	for (e = LIST_HEAD(vs->rs); e; ELEMENT_NEXT(e)) {
 		rs = ELEMENT_DATA(e);
@@ -156,7 +162,8 @@ init_service_rs(virtual_server * vs)
 		 * later upon healthchecks recovery (if ever).
 		 */
 		if (vs->alpha) {
-			UNSET_ALIVE(rs);
+			if (! rs->reloaded)
+				UNSET_ALIVE(rs);
 			continue;
 		}
 		if (!ISALIVE(rs)) {
@@ -177,7 +184,7 @@ init_service_rs(virtual_server * vs)
 
 /* Set a virtualserver IPVS rules */
 static int
-init_service_vs(virtual_server * vs)
+init_service_vs(virtual_server_t * vs)
 {
 	/* Init the VS root */
 	if (!ISALIVE(vs) || vs->vsgname) {
@@ -189,11 +196,16 @@ init_service_vs(virtual_server * vs)
 
 	/* Processing real server queue */
 	if (!LIST_ISEMPTY(vs->rs)) {
-		if (vs->alpha)
+		if (vs->alpha && ! vs->reloaded)
 			vs->quorum_state = DOWN;
 		if (!init_service_rs(vs))
 			return 0;
 	}
+
+	/* if the service was reloaded, we may have got/lost quorum due to quorum setting changed */
+	if (vs->reloaded)
+		update_quorum_state(vs);
+
 	return 1;
 }
 
@@ -203,7 +215,7 @@ init_services(void)
 {
 	element e;
 	list l = check_data->vs;
-	virtual_server *vs;
+	virtual_server_t *vs;
 
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
 		vs = ELEMENT_DATA(e);
@@ -215,18 +227,17 @@ init_services(void)
 
 /* add or remove _alive_ real servers from a virtual server */
 void
-perform_quorum_state(virtual_server *vs, int add)
+perform_quorum_state(virtual_server_t *vs, int add)
 {
 	element e;
-	real_server *rs;
+	real_server_t *rs;
 
 	if (LIST_ISEMPTY(vs->rs))
 		return;
 
-	log_message(LOG_INFO, "%s the pool for VS [%s]:%d"
+	log_message(LOG_INFO, "%s the pool for VS %s"
 			    , add?"Adding alive servers to":"Removing alive servers from"
-			    , (vs->vsgname) ? vs->vsgname : inet_sockaddrtos(&vs->addr)
-			    , ntohs(inet_sockaddrport(&vs->addr)));
+			    , FMT_VS(vs));
 	for (e = LIST_HEAD(vs->rs); e; ELEMENT_NEXT(e)) {
 		rs = ELEMENT_DATA(e);
 		if (!ISALIVE(rs)) /* We only handle alive servers */
@@ -239,28 +250,26 @@ perform_quorum_state(virtual_server *vs, int add)
 }
 
 /* set quorum state depending on current weight of real servers */
-void
-update_quorum_state(virtual_server * vs)
+static void
+update_quorum_state(virtual_server_t * vs)
 {
-	char rsip[INET6_ADDRSTRLEN];
+	long unsigned weight_sum = weigh_live_realservers(vs);
 
 	/* If we have just gained quorum, it's time to consider notify_up. */
 	if (vs->quorum_state == DOWN &&
-	    weigh_live_realservers(vs) >= vs->quorum + vs->hysteresis) {
+	    weight_sum >= vs->quorum + vs->hysteresis) {
 		vs->quorum_state = UP;
-		log_message(LOG_INFO, "Gained quorum %lu+%lu=%lu <= %u for VS [%s]:%d"
+		log_message(LOG_INFO, "Gained quorum %lu+%lu=%lu <= %u for VS %s"
 				    , vs->quorum
 				    , vs->hysteresis
 				    , vs->quorum + vs->hysteresis
-				    , weigh_live_realservers(vs)
-				    , (vs->vsgname) ? vs->vsgname : inet_sockaddrtos(&vs->addr)
-				    , ntohs(inet_sockaddrport(&vs->addr)));
+				    , weight_sum
+				    , FMT_VS(vs));
 		if (vs->s_svr && ISALIVE(vs->s_svr)) {
-			log_message(LOG_INFO, "Removing sorry server [%s]:%d from VS [%s]:%d"
-					    , inet_sockaddrtos2(&vs->s_svr->addr, rsip)
-					    , ntohs(inet_sockaddrport(&vs->s_svr->addr))
-					    , (vs->vsgname) ? vs->vsgname : inet_sockaddrtos(&vs->addr)
-					    , ntohs(inet_sockaddrport(&vs->addr)));
+			log_message(LOG_INFO, "%s sorry server %s from VS %s"
+					    , (vs->s_svr->inhibit ? "Disabling" : "Removing")
+					    , FMT_RS(vs->s_svr)
+					    , FMT_VS(vs));
 
 			ipvs_cmd(LVS_CMD_DEL_DEST, check_data->vs_group, vs, vs->s_svr);
 			vs->s_svr->alive = 0;
@@ -269,10 +278,9 @@ update_quorum_state(virtual_server * vs)
 			perform_quorum_state(vs, 1);
 		}
 		if (vs->quorum_up) {
-			log_message(LOG_INFO, "Executing [%s] for VS [%s]:%d"
+			log_message(LOG_INFO, "Executing [%s] for VS %s"
 					    , vs->quorum_up
-					    , (vs->vsgname) ? vs->vsgname : inet_sockaddrtos(&vs->addr)
-					    , ntohs(inet_sockaddrport(&vs->addr)));
+					    , FMT_VS(vs));
 			notify_exec(vs->quorum_up);
 		}
 #ifdef _WITH_SNMP_
@@ -284,29 +292,28 @@ update_quorum_state(virtual_server * vs)
 	/* If we have just lost quorum for the VS, we need to consider
 	 * VS notify_down and sorry_server cases
 	 */
-	if (vs->quorum_state == UP &&
-	    weigh_live_realservers(vs) < vs->quorum - vs->hysteresis) {
+	if (vs->quorum_state == UP && (
+		!weight_sum ||
+	    weight_sum < vs->quorum - vs->hysteresis)
+	) {
 		vs->quorum_state = DOWN;
-		log_message(LOG_INFO, "Lost quorum %lu-%lu=%lu > %u for VS [%s]:%d"
+		log_message(LOG_INFO, "Lost quorum %lu-%lu=%lu > %u for VS %s"
 				    , vs->quorum
 				    , vs->hysteresis
 				    , vs->quorum - vs->hysteresis
-				    , weigh_live_realservers(vs)
-				    , (vs->vsgname) ? vs->vsgname : inet_sockaddrtos(&vs->addr)
-				    , ntohs(inet_sockaddrport(&vs->addr)));
+				    , weight_sum
+				    , FMT_VS(vs));
 		if (vs->quorum_down) {
-			log_message(LOG_INFO, "Executing [%s] for VS [%s]:%d"
+			log_message(LOG_INFO, "Executing [%s] for VS %s"
 					    , vs->quorum_down
-					    , (vs->vsgname) ? vs->vsgname : inet_sockaddrtos(&vs->addr)
-					    , ntohs(inet_sockaddrport(&vs->addr)));
+					    , FMT_VS(vs));
 			notify_exec(vs->quorum_down);
 		}
 		if (vs->s_svr) {
-			log_message(LOG_INFO, "Adding sorry server [%s]:%d to VS [%s]:%d"
-					    , inet_sockaddrtos2(&vs->s_svr->addr, rsip)
-					    , ntohs(inet_sockaddrport(&vs->s_svr->addr))
-					    , (vs->vsgname) ? vs->vsgname : inet_sockaddrtos(&vs->addr)
-					    , ntohs(inet_sockaddrport(&vs->addr)));
+			log_message(LOG_INFO, "%s sorry server %s to VS %s"
+					    , (vs->s_svr->inhibit ? "Enabling" : "Adding")
+					    , FMT_RS(vs->s_svr)
+					    , FMT_VS(vs));
 
 			/* the sorry server is now up in the pool, we flag it alive */
 			ipvs_cmd(LVS_CMD_ADD_DEST, check_data->vs_group, vs, vs->s_svr);
@@ -324,10 +331,8 @@ update_quorum_state(virtual_server * vs)
 
 /* manipulate add/remove rs according to alive state */
 void
-perform_svr_state(int alive, virtual_server * vs, real_server * rs)
+perform_svr_state(int alive, virtual_server_t * vs, real_server_t * rs)
 {
-	char rsip[INET6_ADDRSTRLEN];
-
 	/*
 	 * | ISALIVE(rs) | alive | context
 	 * | 0           | 0     | first check failed under alpha mode, unreachable here
@@ -336,24 +341,20 @@ perform_svr_state(int alive, virtual_server * vs, real_server * rs)
 	 * | 1           | 1     | first check succeeded w/o alpha mode, unreachable here
 	 */
 	if (!ISALIVE(rs) && alive) {
-		log_message(LOG_INFO, "%s service [%s]:%d to VS [%s]:%d"
+		log_message(LOG_INFO, "%s service %s to VS %s"
 				    , (rs->inhibit) ? "Enabling" : "Adding"
-				    , inet_sockaddrtos2(&rs->addr, rsip)
-				    , ntohs(inet_sockaddrport(&rs->addr))
-				    , (vs->vsgname) ? vs->vsgname : inet_sockaddrtos(&vs->addr)
-				    , ntohs(inet_sockaddrport(&vs->addr)));
+				    , FMT_RS(rs)
+				    , FMT_VS(vs));
 		/* Add only if we have quorum or no sorry server */
 		if (vs->quorum_state == UP || !vs->s_svr || !ISALIVE(vs->s_svr)) {
 			ipvs_cmd(LVS_CMD_ADD_DEST, check_data->vs_group, vs, rs);
 		}
 		rs->alive = alive;
 		if (rs->notify_up) {
-			log_message(LOG_INFO, "Executing [%s] for service [%s]:%d in VS [%s]:%d"
+			log_message(LOG_INFO, "Executing [%s] for service %s in VS %s"
 					    , rs->notify_up
-					    , inet_sockaddrtos2(&rs->addr, rsip)
-					    , ntohs(inet_sockaddrport(&rs->addr))
-					    , (vs->vsgname) ? vs->vsgname : inet_sockaddrtos(&vs->addr)
-					    , ntohs(inet_sockaddrport(&vs->addr)));
+					    , FMT_RS(rs)
+					    , FMT_VS(vs));
 			notify_exec(rs->notify_up);
 		}
 #ifdef _WITH_SNMP_
@@ -365,12 +366,10 @@ perform_svr_state(int alive, virtual_server * vs, real_server * rs)
 	}
 
 	if (ISALIVE(rs) && !alive) {
-		log_message(LOG_INFO, "%s service [%s]:%d from VS [%s]:%d"
+		log_message(LOG_INFO, "%s service %s from VS %s"
 				    , (rs->inhibit) ? "Disabling" : "Removing"
-				    , inet_sockaddrtos2(&rs->addr, rsip)
-				    , ntohs(inet_sockaddrport(&rs->addr))
-				    , (vs->vsgname) ? vs->vsgname : inet_sockaddrtos(&vs->addr)
-				    , ntohs(inet_sockaddrport(&vs->addr)));
+				    , FMT_RS(rs)
+				    , FMT_VS(vs));
 
 		/* server is down, it is removed from the LVS realserver pool
 		 * Remove only if we have quorum or no sorry server
@@ -380,12 +379,10 @@ perform_svr_state(int alive, virtual_server * vs, real_server * rs)
 		}
 		rs->alive = alive;
 		if (rs->notify_down) {
-			log_message(LOG_INFO, "Executing [%s] for service [%s]:%d in VS [%s]:%d"
+			log_message(LOG_INFO, "Executing [%s] for service %s in VS %s"
 					    , rs->notify_down
-					    , inet_sockaddrtos2(&rs->addr, rsip)
-					    , ntohs(inet_sockaddrport(&rs->addr))
-					    , (vs->vsgname) ? vs->vsgname : inet_sockaddrtos(&vs->addr)
-					    , ntohs(inet_sockaddrport(&vs->addr)));
+					    , FMT_RS(rs)
+					    , FMT_VS(vs));
 			notify_exec(rs->notify_down);
 		}
 #ifdef _WITH_SNMP_
@@ -399,19 +396,15 @@ perform_svr_state(int alive, virtual_server * vs, real_server * rs)
 
 /* Store new weight in real_server struct and then update kernel. */
 void
-update_svr_wgt(int weight, virtual_server * vs, real_server * rs)
+update_svr_wgt(int weight, virtual_server_t * vs, real_server_t * rs)
 {
-	char rsip[INET6_ADDRSTRLEN];
-
 	if (weight != rs->weight) {
-		log_message(LOG_INFO, "Changing weight from %d to %d for %s service [%s]:%d of VS [%s]:%d"
+		log_message(LOG_INFO, "Changing weight from %d to %d for %s service %s of VS %s"
 				    , rs->weight
 				    , weight
 				    , ISALIVE(rs) ? "active" : "inactive"
-				    , inet_sockaddrtos2(&rs->addr, rsip)
-				    , ntohs(inet_sockaddrport(&rs->addr))
-				    , (vs->vsgname) ? vs->vsgname : inet_sockaddrtos(&vs->addr)
-				    , ntohs(inet_sockaddrport(&vs->addr)));
+				    , FMT_RS(rs)
+				    , FMT_VS(vs));
 		rs->weight = weight;
 		/*
 		 * Have weight change take effect now only if rs is in
@@ -428,7 +421,7 @@ update_svr_wgt(int weight, virtual_server * vs, real_server * rs)
 
 /* Test if realserver is marked UP for a specific checker */
 int
-svr_checker_up(checker_id_t cid, real_server *rs)
+svr_checker_up(checker_id_t cid, real_server_t *rs)
 {
 	element e;
 	list l = rs->failed_checkers;
@@ -450,7 +443,7 @@ svr_checker_up(checker_id_t cid, real_server *rs)
 
 /* Update checker's state */
 void
-update_svr_checker_state(int alive, checker_id_t cid, virtual_server *vs, real_server *rs)
+update_svr_checker_state(int alive, checker_id_t cid, virtual_server_t *vs, real_server_t *rs)
 {
 	element e;
 	list l = rs->failed_checkers;
@@ -487,10 +480,10 @@ update_svr_checker_state(int alive, checker_id_t cid, virtual_server *vs, real_s
 
 /* Check if a vsg entry is in new data */
 static int
-vsge_exist(virtual_server_group_entry *vsg_entry, list l)
+vsge_exist(virtual_server_group_entry_t *vsg_entry, list l)
 {
 	element e;
-	virtual_server_group_entry *vsge;
+	virtual_server_group_entry_t *vsge;
 
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
 		vsge = ELEMENT_DATA(e);
@@ -510,9 +503,9 @@ vsge_exist(virtual_server_group_entry *vsg_entry, list l)
 
 /* Clear the diff vsge of old group */
 static int
-clear_diff_vsge(list old, list new, virtual_server * old_vs)
+clear_diff_vsge(list old, list new, virtual_server_t * old_vs)
 {
-	virtual_server_group_entry *vsge;
+	virtual_server_group_entry_t *vsge;
 	element e;
 
 	for (e = LIST_HEAD(old); e; ELEMENT_NEXT(e)) {
@@ -535,10 +528,10 @@ clear_diff_vsge(list old, list new, virtual_server * old_vs)
 
 /* Clear the diff vsg of the old vs */
 static int
-clear_diff_vsg(virtual_server * old_vs)
+clear_diff_vsg(virtual_server_t * old_vs)
 {
-	virtual_server_group *old;
-	virtual_server_group *new;
+	virtual_server_group_t *old;
+	virtual_server_group_t *new;
 
 	/* Fetch group */
 	old = ipvs_get_group_by_name(old_vs->vsgname, old_check_data->vs_group);
@@ -555,17 +548,17 @@ clear_diff_vsg(virtual_server * old_vs)
 	return 1;
 }
 
-/* Check if a vs exist in new data */
-static int
-vs_exist(virtual_server * old_vs)
+/* Check if a vs exist in new data and returns pointer to it */
+static virtual_server_t*
+vs_exist(virtual_server_t * old_vs)
 {
 	element e;
 	list l = check_data->vs;
-	virtual_server *vs;
-	virtual_server_group *vsg;
+	virtual_server_t *vs;
+	virtual_server_group_t *vsg;
 
 	if (LIST_ISEMPTY(l))
-		return 0;
+		return NULL;
 
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
 		vs = ELEMENT_DATA(e);
@@ -575,29 +568,29 @@ vs_exist(virtual_server * old_vs)
 				vsg = ipvs_get_group_by_name(old_vs->vsgname,
 							    check_data->vs_group);
 				if (!vsg)
-					return 0;
+					return NULL;
 				else
 					if (!clear_diff_vsg(old_vs))
-						return 0;	
+						return NULL;
 			}
 
 			/*
 			 * Exist so set alive.
 			 */
 			SET_ALIVE(vs);
-			return 1;
+			return vs;
 		}
 	}
 
-	return 0;
+	return NULL;
 }
 
 /* Check if rs is in new vs data */
 static int
-rs_exist(real_server * old_rs, list l)
+rs_exist(real_server_t * old_rs, list l)
 {
 	element e;
-	real_server *rs;
+	real_server_t *rs;
 
 	if (LIST_ISEMPTY(l))
 		return 0;
@@ -622,11 +615,11 @@ rs_exist(real_server * old_rs, list l)
 
 /* get rs list for a specific vs */
 static list
-get_rs_list(virtual_server * vs)
+get_rs_list(virtual_server_t * vs)
 {
 	element e;
 	list l = check_data->vs;
-	virtual_server *vsvr;
+	virtual_server_t *vsvr;
 
 	if (LIST_ISEMPTY(l))
 		return NULL;
@@ -643,37 +636,33 @@ get_rs_list(virtual_server * vs)
 
 /* Clear the diff rs of the old vs */
 static int
-clear_diff_rs(virtual_server * old_vs)
+clear_diff_rs(list old_vs_group, virtual_server_t * old_vs)
 {
 	element e;
 	list l = old_vs->rs;
 	list new = get_rs_list(old_vs);
-	real_server *rs;
-	char rsip[INET6_ADDRSTRLEN];
+	real_server_t *rs;
 
 	/* If old vs didn't own rs then nothing return */
 	if (LIST_ISEMPTY(l))
 		return 1;
 
+	/* remove RS from old vs which are not found in new vs */
+	list rs_to_remove = alloc_list (NULL, NULL);
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
 		rs = ELEMENT_DATA(e);
 		if (!rs_exist(rs, new)) {
 			/* Reset inhibit flag to delete inhibit entries */
-			log_message(LOG_INFO, "service [%s]:%d no longer exist"
-					    , inet_sockaddrtos(&rs->addr)
-					    , ntohs(inet_sockaddrport(&rs->addr)));
-			log_message(LOG_INFO, "Removing service [%s]:%d from VS [%s]:%d"
-					    , inet_sockaddrtos2(&rs->addr, rsip)
-					    , ntohs(inet_sockaddrport(&rs->addr))
-					    , (old_vs->vsgname) ? old_vs->vsgname : inet_sockaddrtos(&old_vs->addr)
-					    , ntohs(inet_sockaddrport(&old_vs->addr)));
+			log_message(LOG_INFO, "service %s no longer exist"
+					    , FMT_RS(rs));
 			rs->inhibit = 0;
-			if (!ipvs_cmd(LVS_CMD_DEL_DEST, check_data->vs_group, old_vs, rs))
-				return 0;
+			list_add (rs_to_remove, rs);
 		}
 	}
+	int ret = clear_service_rs (old_vs_group, old_vs, rs_to_remove);
+	free_list (rs_to_remove);
 
-	return 1;
+	return ret;
 }
 
 /* When reloading configuration, remove negative diff entries */
@@ -682,7 +671,7 @@ clear_diff_services(void)
 {
 	element e;
 	list l = old_check_data->vs;
-	virtual_server *vs;
+	virtual_server_t *vs;
 
 	/* If old config didn't own vs then nothing return */
 	if (LIST_ISEMPTY(l))
@@ -701,16 +690,18 @@ clear_diff_services(void)
 				log_message(LOG_INFO, "Removing Virtual Server Group [%s]"
 						    , vs->vsgname);
 			else
-				log_message(LOG_INFO, "Removing Virtual Server [%s]:%d"
-						    , inet_sockaddrtos(&vs->addr)
-						    , ntohs(inet_sockaddrport(&vs->addr)));
+				log_message(LOG_INFO, "Removing Virtual Server %s"
+						    , FMT_VS(vs));
 
 			/* Clear VS entry */
 			if (!clear_service_vs(old_check_data->vs_group, vs))
 				return 0;
 		} else {
 			/* If vs exist, perform rs pool diff */
-			if (!clear_diff_rs(vs))
+			/* omega = 0 must not prevent the notifiers from being called,
+			   because the VS still exists in new configuration */
+			vs->omega = 1;
+			if (!clear_diff_rs(old_check_data->vs_group, vs))
 				return 0;
 			if (vs->s_svr)
 				if (ISALIVE(vs->s_svr))
@@ -724,3 +715,52 @@ clear_diff_services(void)
 
 	return 1;
 }
+
+/* When reloading configuration, copy still alive RS/VS alive/set attributes into corresponding new config items */
+int
+copy_srv_states (void)
+{
+	element e;
+	list l = old_check_data->vs;
+	virtual_server_t *old_vs, *new_vs;
+
+	/* If old config didn't own vs then nothing return */
+	if (LIST_ISEMPTY(l))
+		return 1;
+
+	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
+		old_vs = ELEMENT_DATA(e);
+		new_vs = vs_exist (old_vs);
+		if (new_vs) {
+			/* copy quorum_state field of VS */
+			new_vs->quorum_state = old_vs->quorum_state;
+			new_vs->reloaded = 1;
+
+			list old_rsl = old_vs->rs;
+			list new_rsl = new_vs->rs;
+			if (LIST_ISEMPTY(old_rsl) || LIST_ISEMPTY (new_rsl))
+				continue;
+			element oe, ne;
+			real_server_t *old_rs, *new_rs;
+			/* iterate over equal rs */
+			for (oe = LIST_HEAD(old_rsl); oe; ELEMENT_NEXT (oe)) {
+				old_rs = ELEMENT_DATA(oe);
+				for (ne = LIST_HEAD(new_rsl); ne; ELEMENT_NEXT(ne)) {
+					new_rs = ELEMENT_DATA(ne);
+					if (RS_ISEQ (old_rs, new_rs)) {
+						/* copy alive, set fields of RS */
+						new_rs->alive = old_rs->alive;
+						new_rs->set = old_rs->set;
+						new_rs->reloaded = 1;
+						if (new_rs->alive) {
+							/* clear failed_checkers list */
+							free_list_elements(new_rs->failed_checkers);
+						}
+						break;
+					}
+				}
+			}
+		}
+	}
+	return 0;
+}
