@@ -30,6 +30,7 @@
 #include "memory.h"
 #include "smtp.h"
 #include "utils.h"
+#include "logger.h"
 
 /* data handlers */
 /* Global def handlers */
@@ -79,6 +80,30 @@ email_handler(vector_t *strvec)
 
 	free_strvec(email_vec);
 }
+static void
+vrrp_mcast_group4_handler(vector_t *strvec)
+{
+	struct sockaddr_storage *mcast = &global_data->vrrp_mcast_group4;
+	int ret;
+
+	ret = inet_stosockaddr(vector_slot(strvec, 1), 0, mcast);
+	if (ret < 0) {
+		log_message(LOG_ERR, "Configuration error: Cant parse vrrp_mcast_group4 [%s]. Skipping"
+				   , vector_slot(strvec, 1));
+	}
+}
+static void
+vrrp_mcast_group6_handler(vector_t *strvec)
+{
+	struct sockaddr_storage *mcast = &global_data->vrrp_mcast_group6;
+	int ret;
+
+	ret = inet_stosockaddr(vector_slot(strvec, 1), 0, mcast);
+	if (ret < 0) {
+		log_message(LOG_ERR, "Configuration error: Cant parse vrrp_mcast_group6 [%s]. Skipping"
+				   , vector_slot(strvec, 1));
+	}
+}
 #ifdef _WITH_SNMP_
 static void
 trap_handler(vector_t *strvec)
@@ -99,6 +124,8 @@ global_init_keywords(void)
 	install_keyword("smtp_server", &smtpip_handler);
 	install_keyword("smtp_connect_timeout", &smtpto_handler);
 	install_keyword("notification_email", &email_handler);
+	install_keyword("vrrp_mcast_group4", &vrrp_mcast_group4_handler);
+	install_keyword("vrrp_mcast_group6", &vrrp_mcast_group6_handler);
 #ifdef _WITH_SNMP_
 	install_keyword("enable_traps", &trap_handler);
 #endif
