@@ -27,17 +27,29 @@
 #include "check_data.h"
 #include "scheduler.h"
 
+/* connection options structure definition */
+typedef struct _conn_opts {
+	struct sockaddr_storage		dst;
+	struct sockaddr_storage		bindto;
+	unsigned int			connection_to; /* connection time-out */
+#ifdef _WITH_SO_MARK_
+	unsigned int			fwmark; /* to mark packets going out of the socket using SO_MARK */
+#endif
+} conn_opts_t;
+
 /* Checkers structure definition */
 typedef struct _checker {
-	void (*free_func) (void *);
-	void (*dump_func) (void *);
-	int (*launch) (struct _thread *);
-	int (*plugin_launch) (void *);
-	virtual_server *vs;	/* pointer to the checker thread virtualserver */
-	real_server *rs;	/* pointer to the checker thread realserver */
-	void *data;
-	checker_id_t id;	/* Checker identifier */
-	int enabled;		/* Activation flag */
+	void				(*free_func) (void *);
+	void				(*dump_func) (void *);
+	int				(*launch) (struct _thread *);
+	int				(*plugin_launch) (void *);
+	virtual_server_t		*vs;	/* pointer to the checker thread virtualserver */
+	real_server_t			*rs;	/* pointer to the checker thread realserver */
+	void				*data;
+	checker_id_t			id;	/* Checker identifier */
+	int				enabled;/* Activation flag */
+	conn_opts_t			*co; /* connection options */
+	long				warmup;	/* max random timeout to start checker */
 } checker_t;
 
 /* Checkers queue */
@@ -45,8 +57,11 @@ extern list checkers_queue;
 
 /* utility macro */
 #define CHECKER_ARG(X) ((X)->data)
+#define CHECKER_CO(X) (((checker_t *)X)->co)
 #define CHECKER_DATA(X) (((checker_t *)X)->data)
-#define CHECKER_GET() (CHECKER_DATA(LIST_TAIL_DATA(checkers_queue)))
+#define CHECKER_GET_CURRENT() (LIST_TAIL_DATA(checkers_queue))
+#define CHECKER_GET() (CHECKER_DATA(CHECKER_GET_CURRENT()))
+#define CHECKER_GET_CO() (((checker_t *)CHECKER_GET_CURRENT())->co)
 #define CHECKER_VALUE_INT(X) (atoi(vector_slot(X,1)))
 #define CHECKER_VALUE_STRING(X) (set_value(X))
 #define CHECKER_VHOST(C) (VHOST((C)->vs))
@@ -54,16 +69,22 @@ extern list checkers_queue;
 #define CHECKER_ENABLE(C)  ((C)->enabled = 1)
 #define CHECKER_DISABLE(C) ((C)->enabled = 0)
 #define CHECKER_HA_SUSPEND(C) ((C)->vs->ha_suspend)
+#define CHECKER_NEW_CO() ((conn_opts_t *) MALLOC(sizeof (conn_opts_t)))
+#define FMT_CHK(C) FMT_RS((C)->rs)
 
 /* Prototypes definition */
 extern void init_checkers_queue(void);
+extern void dump_conn_opts (conn_opts_t *);
 extern void queue_checker(void (*free_func) (void *), void (*dump_func) (void *)
 			  , int (*launch) (thread_t *)
-			  , void *);
+			  , void *
+			  , conn_opts_t *);
 extern void dump_checkers_queue(void);
 extern void free_checkers_queue(void);
 extern void register_checkers_thread(void);
 extern void install_checkers_keyword(void);
+extern void install_connect_keywords(void);
+extern void warmup_handler(vector_t *);
 extern void update_checker_activity(sa_family_t, void *, int);
 extern void checker_set_dst(struct sockaddr_storage *);
 extern void checker_set_dst_port(struct sockaddr_storage *, uint16_t);
