@@ -17,7 +17,7 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2011 Alexandre Cassen, <acassen@linux-vs.org>
+ * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@gmail.com>
  */
 
 #ifndef _CHECK_PARSER_H
@@ -27,6 +27,6 @@
 #include "vector.h"
 
 /* Prototypes */
-extern vector check_init_keywords(void);
+extern vector_t *check_init_keywords(void);
 
 #endif
