@@ -31,6 +31,7 @@
 #include "check_data.h"
 #include "scheduler.h"
 #include "list.h"
+#include "check_api.h"
 
 #define SMTP_BUFF_MAX		512
 
@@ -45,33 +46,33 @@
 #define SMTP_DEFAULT_PORT	25
 
 /* Per host configuration structure  */
-typedef struct _smtp_host {
-	struct sockaddr_storage dst;
-	struct sockaddr_storage bindto;
-} smtp_host_t;
+typedef conn_opts_t smtp_host_t;
 
 /* Checker argument structure  */
 typedef struct _smtp_checker {
 	/* non per host config data goes here */
-	char *helo_name;
-	long timeout;
-	long db_retry;
-	int retry;
-	int attempts;
-	int  host_ctr;
-	smtp_host_t *host_ptr;
+	char				*helo_name;
+	long				timeout;
+	long				db_retry;
+	int				retry;
+	int				attempts;
+	int				host_ctr;
+	smtp_host_t			*host_ptr;
 
 	/* data buffer */
-	char buff[SMTP_BUFF_MAX];
-	int buff_ctr;
-	int (*buff_cb) (thread_t *);
+	char				buff[SMTP_BUFF_MAX];
+	int				buff_ctr;
+	int				(*buff_cb) (thread_t *);
 
-	int state;
+	int				state;
 
 	/* list holding the host config data */
-	list host;
+	list				host;
 } smtp_checker_t;
 
+/* macro utility */
+#define FMT_SMTP_RS(H) (inet_sockaddrtopair (&(H)->dst))
+
 /* Prototypes defs */
 extern void install_smtp_check_keyword(void);
 
