@@ -38,26 +38,26 @@
 				   -- rfc2402.2.2 */
 #define IPPROTO_IPSEC_AH 51	/* IP protocol number -- rfc2402.2 */
 
-typedef struct {		/* rfc2402.2 */
-	uint8_t next_header;	/* Next header field */
-	uint8_t payload_len;	/* Payload Lenght */
-	uint16_t reserved;	/* Reserved field */
-	uint32_t spi;		/* Security Parameter Index */
-	uint32_t seq_number;	/* Sequence number */
-	uint32_t auth_data[3];	/* Authentication data 128-bit MD5 digest trucated
-				   => HMAC_MD5_TRUNC*8/32 */
-} ipsec_ah;
+typedef struct _ipsec_ah {				/* rfc2402.2 */
+	uint8_t			next_header;	/* Next header field */
+	uint8_t			payload_len;	/* Payload Lenght */
+	uint16_t		reserved;	/* Reserved field */
+	uint32_t		spi;		/* Security Parameter Index */
+	uint32_t		seq_number;	/* Sequence number */
+	uint32_t		auth_data[3];	/* Authentication data 128-bit MD5 digest trucated
+						   => HMAC_MD5_TRUNC*8/32 */
+} ipsec_ah_t;
 
 typedef struct {		/* rfc2402.3.3.3.1.1.1 */
-	u_int8_t tos;
-	u_int16_t frag_off;
-	u_int16_t check;
+	uint8_t			tos;
+	uint16_t		frag_off;
+	uint16_t		check;
 } ICV_mutable_fields;		/* We need to zero this fields to compute the ICV */
 
-typedef struct {
-	int cycle;
-	uint32_t seq_number;
-} seq_counter;
+typedef struct _seq_counter {
+	int			cycle;
+	uint32_t		seq_number;
+} seq_counter_t;
 
 extern void hmac_md5(unsigned char *, int, unsigned char *, int, unsigned char *);
 
