@@ -29,15 +29,15 @@
 
 /* VRID hash table */
 void
-alloc_vrrp_bucket(vrrp_rt *vrrp)
+alloc_vrrp_bucket(vrrp_t *vrrp)
 {
 	list_add(&vrrp_data->vrrp_index[vrrp->vrid], vrrp);
 }
 
-vrrp_rt *
+vrrp_t *
 vrrp_index_lookup(const int vrid, const int fd)
 {
-	vrrp_rt *vrrp;
+	vrrp_t *vrrp;
 	element e;
 	list l = &vrrp_data->vrrp_index[vrid];
 
@@ -71,30 +71,57 @@ vrrp_index_lookup(const int vrid, const int fd)
 
 /* FD hash table */
 void
-alloc_vrrp_fd_bucket(vrrp_rt *vrrp)
+alloc_vrrp_fd_bucket(vrrp_t *vrrp)
 {
 	/* We use a mod key plus 1 */
 	list_add(&vrrp_data->vrrp_index_fd[vrrp->fd_in%1024 + 1], vrrp);
 }
 
 void
-remove_vrrp_fd_bucket(vrrp_rt *vrrp)
+remove_vrrp_fd_bucket(vrrp_t *vrrp)
 {
 	list l = &vrrp_data->vrrp_index_fd[vrrp->fd_in%1024 + 1];
 	list_del(l, vrrp);
 }
 
-void set_vrrp_fd_bucket(int old_fd, vrrp_rt *vrrp)
+void set_vrrp_fd_bucket(int old_fd, vrrp_t *vrrp)
 {
-	vrrp_rt *vrrp_ptr;
+	vrrp_t *vrrp_ptr;
 	element e;
+	element next;
 	list l = &vrrp_data->vrrp_index_fd[old_fd%1024 + 1];
 
-	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
+	/* Release old stalled entries */
+	for (e = LIST_HEAD(l); e; e = next) {
+		next = e->next;
 		vrrp_ptr =  ELEMENT_DATA(e);
-		if (IF_INDEX(vrrp_ptr->ifp) == IF_INDEX(vrrp->ifp)) {
+		if (vrrp_ptr->fd_in == old_fd) {
+			if (e->prev)
+				e->prev->next = e->next;
+			else
+				 l->head = e->next;
+
+			if (e->next)
+				e->next->prev = e->prev;
+			else
+				l->tail = e->prev;
+			l->count--;
+			FREE(e);
+		}
+	}
+	if (LIST_ISEMPTY(l))
+		l->head = l->tail = NULL;
+
+	/* Hash refreshed entries */
+	l = vrrp_data->vrrp;
+	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
+		vrrp_ptr = ELEMENT_DATA(e);
+
+		if (vrrp_ptr->fd_in == old_fd) {
+			/* Update new hash */
 			vrrp_ptr->fd_in = vrrp->fd_in;
 			vrrp_ptr->fd_out = vrrp->fd_out;
+			alloc_vrrp_fd_bucket(vrrp_ptr);
 		}
 	}
 }
