@@ -44,17 +44,17 @@
 #include "utils.h"
 
 /* Global vars */
-struct nl_handle nl_kernel;	/* Kernel reflection channel */
-struct nl_handle nl_cmd;	/* Command channel */
+nl_handle_t nl_kernel;	/* Kernel reflection channel */
+nl_handle_t nl_cmd;	/* Command channel */
 
-/* Create a socket to netlink interface */
+/* Create a socket to netlink interface_t */
 int
-netlink_socket(struct nl_handle *nl, unsigned long groups)
+netlink_socket(nl_handle_t *nl, unsigned long groups)
 {
 	socklen_t addr_len;
 	int ret;
 
-	memset(nl, 0, sizeof (nl));
+	memset(nl, 0, sizeof (*nl));
 
 	nl->fd = socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
 	if (nl->fd < 0) {
@@ -102,20 +102,27 @@ netlink_socket(struct nl_handle *nl, unsigned long groups)
 
 	nl->seq = time(NULL);
 
+	/* Set default rcvbuf size */
+	if_setsockopt_rcvbuf(&nl->fd, IF_DEFAULT_BUFSIZE);
+	if (nl->fd < 0)
+		return -1;
+
 	return ret;
 }
 
 /* Close a netlink socket */
 int
-netlink_close(struct nl_handle *nl)
+netlink_close(nl_handle_t *nl)
 {
+	/* First of all release pending thread */
+	thread_cancel(nl->thread);
 	close(nl->fd);
 	return 0;
 }
 
 /* Set netlink socket channel as blocking */
 int
-netlink_set_block(struct nl_handle *nl, int *flags)
+netlink_set_block(nl_handle_t *nl, int *flags)
 {
 	if ((*flags = fcntl(nl->fd, F_GETFL, 0)) < 0) {
 		log_message(LOG_INFO, "Netlink: Cannot F_GETFL socket : (%s)",
@@ -133,7 +140,7 @@ netlink_set_block(struct nl_handle *nl, int *flags)
 
 /* Set netlink socket channel as non-blocking */
 int
-netlink_set_nonblock(struct nl_handle *nl, int *flags)
+netlink_set_nonblock(nl_handle_t *nl, int *flags)
 {
 	*flags |= O_NONBLOCK;
 	if (fcntl(nl->fd, F_SETFL, *flags) < 0) {
@@ -237,10 +244,29 @@ netlink_scope_a2n(char *scope)
 	return -1;
 }
 
+/*
+ * Reflect base interface flags on VMAC interface.
+ * VMAC interfaces should never update it own flags, only be reflected
+ * by the base interface flags.
+ */
+static void
+vmac_reflect_flags(struct ifinfomsg *ifi)
+{
+	interface_t *ifp;
+
+	/* find the VMAC interface (if any) */
+	ifp = if_get_by_vmac_base_ifindex(ifi->ifi_index);
+
+	/* if found, reflect base interface flags on VMAC interface */
+	if (ifp) {
+		ifp->flags = ifi->ifi_flags;
+	}
+}
+
 /* Our netlink parser */
 static int
 netlink_parse_info(int (*filter) (struct sockaddr_nl *, struct nlmsghdr *),
-		   struct nl_handle *nl, struct nlmsghdr *n)
+		   nl_handle_t *nl, struct nlmsghdr *n)
 {
 	int status;
 	int ret = 0;
@@ -261,7 +287,7 @@ netlink_parse_info(int (*filter) (struct sockaddr_nl *, struct nlmsghdr *),
 				continue;
 			if (errno == EWOULDBLOCK || errno == EAGAIN)
 				break;
-			log_message(LOG_INFO, "Netlink: Received message overrun");
+			log_message(LOG_INFO, "Netlink: Received message overrun (%m)");
 			continue;
 		}
 
@@ -354,7 +380,7 @@ netlink_talk_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
 
 /* send message to netlink kernel socket, then receive response */
 int
-netlink_talk(struct nl_handle *nl, struct nlmsghdr *n)
+netlink_talk(nl_handle_t *nl, struct nlmsghdr *n)
 {
 	int status;
 	int ret, flags;
@@ -394,7 +420,7 @@ netlink_talk(struct nl_handle *nl, struct nlmsghdr *n)
 
 /* Fetch a specific type information from netlink kernel */
 static int
-netlink_request(struct nl_handle *nl, int family, int type)
+netlink_request(nl_handle_t *nl, int family, int type)
 {
 	int status;
 	struct sockaddr_nl snl;
@@ -430,7 +456,7 @@ netlink_if_link_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
 {
 	struct ifinfomsg *ifi;
 	struct rtattr *tb[IFLA_MAX + 1];
-	interface *ifp;
+	interface_t *ifp;
 	int i, len;
 	char *name;
 
@@ -456,17 +482,27 @@ netlink_if_link_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
 
 	/* Skip it if already exist */
 	ifp = if_get_by_ifname(name);
-	if (ifp)
+	if (ifp) {
+		if (!ifp->vmac) {
+			vmac_reflect_flags(ifi);
+			ifp->flags = ifi->ifi_flags;
+		}
 		return 0;
+	}
 
 	/* Fill the interface structure */
-	ifp = (interface *) MALLOC(sizeof (interface));
+	ifp = (interface_t *) MALLOC(sizeof(interface_t));
 	memcpy(ifp->ifname, name, strlen(name));
 	ifp->ifindex = ifi->ifi_index;
-	ifp->flags = ifi->ifi_flags;
 	ifp->mtu = *(int *) RTA_DATA(tb[IFLA_MTU]);
 	ifp->hw_type = ifi->ifi_type;
 
+	if (!ifp->vmac) {
+		vmac_reflect_flags(ifi);
+		ifp->flags = ifi->ifi_flags;
+		ifp->base_ifindex = ifi->ifi_index;
+	}
+
 	if (tb[IFLA_ADDRESS]) {
 		int hw_addr_len = RTA_PAYLOAD(tb[IFLA_ADDRESS]);
 
@@ -487,7 +523,7 @@ netlink_if_link_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
 		}
 	}
 
-	/* Queue this new interface */
+	/* Queue this new interface_t */
 	if_add_queue(ifp);
 	return 0;
 }
@@ -502,7 +538,7 @@ netlink_if_address_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
 {
 	struct ifaddrmsg *ifa;
 	struct rtattr *tb[IFA_MAX + 1];
-	interface *ifp;
+	interface_t *ifp;
 	int len;
 	void *addr;
 
@@ -522,7 +558,7 @@ netlink_if_address_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
 	memset(tb, 0, sizeof (tb));
 	parse_rtattr(tb, IFA_MAX, IFA_RTA(ifa), len);
 
-	/* Fetch interface */
+	/* Fetch interface_t */
 	ifp = if_get_by_ifindex(ifa->ifa_index);
 	if (!ifp)
 		return 0;
@@ -558,7 +594,7 @@ netlink_if_address_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
 int
 netlink_interface_lookup(void)
 {
-	struct nl_handle nlh;
+	nl_handle_t nlh;
 	int status = 0;
 	int ret, flags;
 
@@ -587,7 +623,7 @@ end_int:
 static int
 netlink_address_lookup(void)
 {
-	struct nl_handle nlh;
+	nl_handle_t nlh;
 	int status = 0;
 	int ret, flags;
 
@@ -625,7 +661,7 @@ netlink_reflect_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
 {
 	struct ifinfomsg *ifi;
 	struct rtattr *tb[IFLA_MAX + 1];
-	interface *ifp;
+	interface_t *ifp;
 	int len;
 
 	ifi = NLMSG_DATA(h);
@@ -646,13 +682,20 @@ netlink_reflect_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
 	if (ifi->ifi_type == ARPHRD_LOOPBACK)
 		return 0;
 
-	/* find the interface */
+	/* find the interface_t */
 	ifp = if_get_by_ifindex(ifi->ifi_index);
 	if (!ifp)
 		return -1;
 
-	/* Update flags */
-	ifp->flags = ifi->ifi_flags;
+	/*
+	 * Update flags.
+	 * VMAC interfaces should never update it own flags, only be reflected
+	 * by the base interface flags.
+	 */
+	if (!ifp->vmac) {
+		vmac_reflect_flags(ifi);
+		ifp->flags = ifi->ifi_flags;
+	}
 
 	return 0;
 }
@@ -682,10 +725,12 @@ netlink_broadcast_filter(struct sockaddr_nl *snl, struct nlmsghdr *h)
 int
 kernel_netlink(thread_t * thread)
 {
+	nl_handle_t *nl = THREAD_ARG(thread);
+
 	if (thread->type != THREAD_READ_TIMEOUT)
-		netlink_parse_info(netlink_broadcast_filter, &nl_kernel, NULL);
-	thread_add_read(master, kernel_netlink, NULL, nl_kernel.fd,
-			NETLINK_TIMER);
+		netlink_parse_info(netlink_broadcast_filter, nl, NULL);
+	nl->thread = thread_add_read(master, kernel_netlink, nl, nl->fd,
+				      NETLINK_TIMER);
 	return 0;
 }
 
@@ -707,8 +752,8 @@ kernel_netlink_init(void)
 
 	if (nl_kernel.fd > 0) {
 		log_message(LOG_INFO, "Registering Kernel netlink reflector");
-		thread_add_read(master, kernel_netlink, NULL, nl_kernel.fd,
-				NETLINK_TIMER);
+		nl_kernel.thread = thread_add_read(master, kernel_netlink, &nl_kernel, nl_kernel.fd,
+						   NETLINK_TIMER);
 	} else
 		log_message(LOG_INFO, "Error while registering Kernel netlink reflector channel");
 
