@@ -58,47 +58,47 @@ vrrp_sync_group_handler(vector_t *strvec)
 static void
 vrrp_group_handler(vector_t *strvec)
 {
-	vrrp_sgroup *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
+	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
 	vgroup->iname = read_value_block();
 }
 static void
 vrrp_gnotify_backup_handler(vector_t *strvec)
 {
-	vrrp_sgroup *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
+	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
 	vgroup->script_backup = set_value(strvec);
 	vgroup->notify_exec = 1;
 }
 static void
 vrrp_gnotify_master_handler(vector_t *strvec)
 {
-	vrrp_sgroup *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
+	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
 	vgroup->script_master = set_value(strvec);
 	vgroup->notify_exec = 1;
 }
 static void
 vrrp_gnotify_fault_handler(vector_t *strvec)
 {
-	vrrp_sgroup *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
+	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
 	vgroup->script_fault = set_value(strvec);
 	vgroup->notify_exec = 1;
 }
 static void
 vrrp_gnotify_handler(vector_t *strvec)
 {
-	vrrp_sgroup *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
+	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
 	vgroup->script = set_value(strvec);
 	vgroup->notify_exec = 1;
 }
 static void
 vrrp_gsmtp_handler(vector_t *strvec)
 {
-	vrrp_sgroup *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
+	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
 	vgroup->smtp_alert = 1;
 }
 static void
 vrrp_gglobal_tracking_handler(vector_t *strvec)
 {
-	vrrp_sgroup *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
+	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
 	vgroup->global_tracking = 1;
 }
 static void
@@ -109,29 +109,37 @@ vrrp_handler(vector_t *strvec)
 static void
 vrrp_vmac_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->vmac = 1;
-	if (!vrrp->mcast_saddr)
-		vrrp->mcast_saddr  = IF_ADDR(vrrp->ifp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp->vmac_flags |= VRRP_VMAC_FL_SET;
+	if (!vrrp->saddr.ss_family && vrrp->family == AF_INET)
+		inet_ip4tosockaddr(IF_ADDR(vrrp->ifp), &vrrp->saddr);
 	if (vector_size(strvec) == 2) {
 		strncpy(vrrp->vmac_ifname, vector_slot(strvec, 1),
 			IFNAMSIZ - 1);
 	} else if (vrrp->vrid) {
 		snprintf(vrrp->vmac_ifname, IFNAMSIZ, "vrrp.%d", vrrp->vrid);
+	} else {
+		return;
 	}
 
-	if (strlen(vrrp->vmac_ifname)) {
-		log_message(LOG_INFO, "vmac_ifname=%s for vrrp_instace %s"
-				    , vrrp->vmac_ifname
-				    , vrrp->iname);
-	}
-	if (vrrp->ifp && !(vrrp->vmac & 2))
-		netlink_link_add_vmac(vrrp);
+	netlink_link_add_vmac(vrrp);
+}
+static void
+vrrp_vmac_xmit_base_handler(vector_t *strvec)
+{
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	if (vrrp->vmac_flags & VRRP_VMAC_FL_SET)
+		vrrp->vmac_flags |= VRRP_VMAC_FL_XMITBASE;
+}
+static void
+vrrp_unicast_peer_handler(vector_t *strvec)
+{
+	alloc_value_block(strvec, alloc_vrrp_unicast_peer);
 }
 static void
 vrrp_native_ipv6_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->family = AF_INET6;
 
 	if (vrrp->auth_type != VRRP_AUTH_NONE)
@@ -141,8 +149,8 @@ static void
 vrrp_state_handler(vector_t *strvec)
 {
 	char *str = vector_slot(strvec, 1);
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp_sgroup *vgroup = vrrp->sync;
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_sgroup_t *vgroup = vrrp->sync;
 
 	if (!strcmp(str, "MASTER")) {
 		vrrp->wantstate = VRRP_STATE_MAST;
@@ -156,11 +164,19 @@ vrrp_state_handler(vector_t *strvec)
 static void
 vrrp_int_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	char *name = vector_slot(strvec, 1);
+
 	vrrp->ifp = if_get_by_ifname(name);
-	if (vrrp->vmac && !(vrrp->vmac & 2))
+	if (!vrrp->ifp) {
+		log_message(LOG_INFO, "Cant find interface %s for vrrp_instance %s !!!"
+				    , name, vrrp->iname);
+		return;
+	}
+
+	if (vrrp->vmac_flags & VRRP_VMAC_FL_SET) {
 		netlink_link_add_vmac(vrrp);
+	}
 }
 static void
 vrrp_track_int_handler(vector_t *strvec)
@@ -175,19 +191,35 @@ vrrp_track_scr_handler(vector_t *strvec)
 static void
 vrrp_dont_track_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->dont_track_primary = 1;
 }
 static void
-vrrp_mcastip_handler(vector_t *strvec)
+vrrp_srcip_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	inet_ston(vector_slot(strvec, 1), &vrrp->mcast_saddr);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	struct sockaddr_storage *saddr = &vrrp->saddr;
+	int ret;
+
+	ret = inet_stosockaddr(vector_slot(strvec, 1), 0, saddr);
+	if (ret < 0) {
+		log_message(LOG_ERR, "Configuration error: VRRP instance[%s] malformed unicast"
+				     " src address[%s]. Skipping..."
+				   , vrrp->iname, vector_slot(strvec, 1));
+		return;
+	}
+
+	if (saddr->ss_family != vrrp->family) {
+		log_message(LOG_ERR, "Configuration error: VRRP instance[%s] and unicast src address"
+				     "[%s] MUST be of the same family !!! Skipping..."
+				   , vrrp->iname, vector_slot(strvec, 1));
+		memset(saddr, 0, sizeof(struct sockaddr_storage));
+	}
 }
 static void
 vrrp_vrid_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->vrid = atoi(vector_slot(strvec, 1));
 
 	if (VRRP_IS_BAD_VID(vrrp->vrid)) {
@@ -196,19 +228,17 @@ vrrp_vrid_handler(vector_t *strvec)
 		       "             must be between 1 & 255. reconfigure !");
 	} else {
 		alloc_vrrp_bucket(vrrp);
-		if (vrrp->vmac && strlen(vrrp->vmac_ifname) == 0) {
-			snprintf(vrrp->vmac_ifname, IFNAMSIZ, "vrrp.%d"
-						  , vrrp->vrid);
-			log_message(LOG_INFO, "vmac_ifname=%s for vrrp_instace %s"
-					    , vrrp->vmac_ifname
-					    , vrrp->iname);
+		if (vrrp->vmac_flags & VRRP_VMAC_FL_SET) {
+			if (strlen(vrrp->vmac_ifname) == 0)
+				snprintf(vrrp->vmac_ifname, IFNAMSIZ, "vrrp.%d", vrrp->vrid);
+			netlink_link_add_vmac(vrrp);
 		}
 	}
 }
 static void
 vrrp_prio_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->effective_priority = vrrp->base_priority = atoi(vector_slot(strvec, 1));
 
 	if (VRRP_IS_BAD_PRIORITY(vrrp->base_priority)) {
@@ -223,7 +253,7 @@ vrrp_prio_handler(vector_t *strvec)
 static void
 vrrp_adv_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->adver_int = atoi(vector_slot(strvec, 1));
 
 	if (VRRP_IS_BAD_ADVERT_INT(vrrp->adver_int)) {
@@ -238,7 +268,7 @@ vrrp_adv_handler(vector_t *strvec)
 static void
 vrrp_debug_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->debug = atoi(vector_slot(strvec, 1));
 
 	if (VRRP_IS_BAD_DEBUG_INT(vrrp->debug)) {
@@ -250,19 +280,19 @@ vrrp_debug_handler(vector_t *strvec)
 static void
 vrrp_nopreempt_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->nopreempt = 1;
 }
 static void	/* backwards compatibility */
 vrrp_preempt_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->nopreempt = 0;
 }
 static void
 vrrp_preempt_delay_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->preempt_delay = atoi(vector_slot(strvec, 1));
 
 	if (VRRP_IS_BAD_PREEMPT_DELAY(vrrp->preempt_delay)) {
@@ -277,62 +307,68 @@ vrrp_preempt_delay_handler(vector_t *strvec)
 static void
 vrrp_notify_backup_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->script_backup = set_value(strvec);
 	vrrp->notify_exec = 1;
 }
 static void
 vrrp_notify_master_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->script_master = set_value(strvec);
 	vrrp->notify_exec = 1;
 }
 static void
 vrrp_notify_fault_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->script_fault = set_value(strvec);
 	vrrp->notify_exec = 1;
 }
 static void
 vrrp_notify_stop_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->script_stop = set_value(strvec);
 	vrrp->notify_exec = 1;
 }
 static void
 vrrp_notify_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->script = set_value(strvec);
 	vrrp->notify_exec = 1;
 }
 static void
 vrrp_smtp_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->smtp_alert = 1;
 }
 static void
 vrrp_lvs_syncd_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->lvs_syncd_if = set_value(strvec);
 }
 static void
 vrrp_garp_delay_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->garp_delay = atoi(vector_slot(strvec, 1)) * TIMER_HZ;
 	if (vrrp->garp_delay < TIMER_HZ)
 		vrrp->garp_delay = TIMER_HZ;
 }
 static void
+vrrp_garp_refresh_handler(vector_t *strvec)
+{
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp->garp_refresh = atoi(vector_slot(strvec, 1)) * TIMER_HZ;
+}
+static void
 vrrp_auth_type_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	char *str = vector_slot(strvec, 1);
 
 	if (!strcmp(str, "AH") && vrrp->family == AF_INET)
@@ -343,7 +379,7 @@ vrrp_auth_type_handler(vector_t *strvec)
 static void
 vrrp_auth_pass_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	char *str = vector_slot(strvec, 1);
 	int max_size = sizeof (vrrp->auth_data);
 	int str_len = strlen(str);
@@ -360,7 +396,7 @@ vrrp_auth_pass_handler(vector_t *strvec)
 static void
 vrrp_vip_handler(vector_t *strvec)
 {
-	vrrp_rt *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	char *buf;
 	char *str = NULL;
 	vector_t *vec = NULL;
@@ -414,13 +450,13 @@ vrrp_script_handler(vector_t *strvec)
 static void
 vrrp_vscript_script_handler(vector_t *strvec)
 {
-	vrrp_script *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
+	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
 	vscript->script = set_value(strvec);
 }
 static void
 vrrp_vscript_interval_handler(vector_t *strvec)
 {
-	vrrp_script *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
+	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
 	vscript->interval = atoi(vector_slot(strvec, 1)) * TIMER_HZ;
 	if (vscript->interval < TIMER_HZ)
 		vscript->interval = TIMER_HZ;
@@ -428,7 +464,7 @@ vrrp_vscript_interval_handler(vector_t *strvec)
 static void
 vrrp_vscript_timeout_handler(vector_t *strvec)
 {
-	vrrp_script *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
+	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
 	vscript->timeout = atoi(vector_slot(strvec, 1)) * TIMER_HZ;
 	if (vscript->timeout < TIMER_HZ)
 		vscript->timeout = TIMER_HZ;
@@ -436,13 +472,13 @@ vrrp_vscript_timeout_handler(vector_t *strvec)
 static void
 vrrp_vscript_weight_handler(vector_t *strvec)
 {
-	vrrp_script *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
+	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
 	vscript->weight = atoi(vector_slot(strvec, 1));
 }
 static void
 vrrp_vscript_rise_handler(vector_t *strvec)
 {
-	vrrp_script *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
+	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
 	vscript->rise = atoi(vector_slot(strvec, 1));
 	if (vscript->rise < 1)
 		vscript->rise = 1;
@@ -450,7 +486,7 @@ vrrp_vscript_rise_handler(vector_t *strvec)
 static void
 vrrp_vscript_fall_handler(vector_t *strvec)
 {
-	vrrp_script *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
+	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
 	vscript->fall = atoi(vector_slot(strvec, 1));
 	if (vscript->fall < 1)
 		vscript->fall = 1;
@@ -477,13 +513,16 @@ vrrp_init_keywords(void)
 	install_keyword("global_tracking", &vrrp_gglobal_tracking_handler);
 	install_keyword_root("vrrp_instance", &vrrp_handler);
 	install_keyword("use_vmac", &vrrp_vmac_handler);
+	install_keyword("vmac_xmit_base", &vrrp_vmac_xmit_base_handler);
+	install_keyword("unicast_peer", &vrrp_unicast_peer_handler);
 	install_keyword("native_ipv6", &vrrp_native_ipv6_handler);
 	install_keyword("state", &vrrp_state_handler);
 	install_keyword("interface", &vrrp_int_handler);
 	install_keyword("dont_track_primary", &vrrp_dont_track_handler);
 	install_keyword("track_interface", &vrrp_track_int_handler);
 	install_keyword("track_script", &vrrp_track_scr_handler);
-	install_keyword("mcast_src_ip", &vrrp_mcastip_handler);
+	install_keyword("mcast_src_ip", &vrrp_srcip_handler);
+	install_keyword("unicast_src_ip", &vrrp_srcip_handler);
 	install_keyword("virtual_router_id", &vrrp_vrid_handler);
 	install_keyword("priority", &vrrp_prio_handler);
 	install_keyword("advert_int", &vrrp_adv_handler);
@@ -502,6 +541,7 @@ vrrp_init_keywords(void)
 	install_keyword("smtp_alert", &vrrp_smtp_handler);
 	install_keyword("lvs_sync_daemon_interface", &vrrp_lvs_syncd_handler);
 	install_keyword("garp_master_delay", &vrrp_garp_delay_handler);
+	install_keyword("garp_master_refresh", &vrrp_garp_refresh_handler);
 	install_keyword("authentication", NULL);
 	install_sublevel();
 	install_keyword("auth_type", &vrrp_auth_type_handler);
