@@ -17,7 +17,7 @@
  *              as published by the Free Software Foundation; either version
  *              2 of the License, or (at your option) any later version.
  *
- * Copyright (C) 2001-2011 Alexandre Cassen, <acassen@linux-vs.org>
+ * Copyright (C) 2001-2012 Alexandre Cassen, <acassen@linux-vs.org>
  */
 
 #ifndef _TIMER_H
@@ -25,29 +25,31 @@
 
 #include <sys/time.h>
 
-typedef struct timeval TIMEVAL;
+typedef struct timeval timeval_t;
 
 /* Global vars */
-extern TIMEVAL time_now;
+extern timeval_t time_now;
 
-/* macro utilities */
-#define TIME_MAX_FORWARD_US 2000000
-#define TIMER_HZ      1000000
-#define TIMER_MAX_SEC 1000
-#define TIMER_SEC(T) ((T).tv_sec)
-#define TIMER_LONG(T) ((T).tv_sec * TIMER_HZ + (T).tv_usec)
-#define TIMER_ISNULL(T) ((T).tv_sec == 0 && (T).tv_usec == 0)
-#define TIMER_RESET(T) (memset(&(T), 0, sizeof(struct timeval)))
+/* Some defines */
+#define TIME_MAX_FORWARD_US	2000000
+#define TIMER_HZ		1000000
+#define TIMER_MAX_SEC		1000
+
+/* Some usefull macros */
+#define timer_sec(T) ((T).tv_sec)
+#define timer_long(T) ((T).tv_sec * TIMER_HZ + (T).tv_usec)
+#define timer_isnull(T) ((T).tv_sec == 0 && (T).tv_usec == 0)
+#define timer_reset(T) (memset(&(T), 0, sizeof(timeval_t)))
 
 /* prototypes */
-extern TIMEVAL timer_now(void);
-extern TIMEVAL set_time_now(void);
-extern TIMEVAL timer_dup(TIMEVAL b);
-extern int timer_cmp(TIMEVAL a, TIMEVAL b);
-extern TIMEVAL timer_sub(TIMEVAL a, TIMEVAL b);
-extern TIMEVAL timer_add_long(TIMEVAL a, long b);
-extern TIMEVAL timer_sub_now(TIMEVAL a);
-extern void timer_dump(TIMEVAL a);
-extern unsigned long timer_tol(TIMEVAL a);
+extern timeval_t timer_now(void);
+extern timeval_t set_time_now(void);
+extern timeval_t timer_dup(timeval_t);
+extern int timer_cmp(timeval_t, timeval_t);
+extern timeval_t timer_sub(timeval_t, timeval_t);
+extern timeval_t timer_add_long(timeval_t, long);
+extern timeval_t timer_sub_now(timeval_t);
+extern void timer_dump(timeval_t);
+extern unsigned long timer_tol(timeval_t);
 
 #endif
